@@ -30,7 +30,6 @@
 #include <ql/methods/finitedifferences/operators/secondderivativeop.hpp>
 #include <ql/methods/finitedifferences/operators/secondordermixedderivativeop.hpp>
 #include <ql/processes/hestonprocess.hpp>
-#include <boost/unordered/unordered_map.hpp>
 #include <cmath>
 #include <utility>
 
@@ -224,7 +223,6 @@ namespace QuantLib {
         return v;
     }
 
-#if !defined(QL_NO_UBLAS_SUPPORT)
     Disposable<std::vector<SparseMatrix> > FdmHestonFwdOp::toMatrixDecomp()
         const {
 
@@ -236,5 +234,5 @@ namespace QuantLib {
 
         return retVal;
     }
-#endif
+
 }
