@@ -53,6 +53,7 @@
 #include "autocovariances.hpp"
 #include "barrieroption.hpp"
 #include "basismodels.hpp"
+#include "basisswapratehelpers.hpp"
 #include "basketoption.hpp"
 #include "batesmodel.hpp"
 #include "bermudanswaption.hpp"
@@ -60,6 +61,7 @@
 #include "blackdeltacalculator.hpp"
 #include "blackformula.hpp"
 #include "bonds.hpp"
+#include "bondforward.hpp"
 #include "brownianbridge.hpp"
 #include "businessdayconventions.hpp"
 #include "calendars.hpp"
@@ -158,6 +160,7 @@
 #include "operators.hpp"
 #include "optimizers.hpp"
 #include "optionletstripper.hpp"
+#include "overnightindexedcoupon.hpp"
 #include "overnightindexedswap.hpp"
 #include "pagodaoption.hpp"
 #include "partialtimebarrieroption.hpp"
@@ -372,6 +375,7 @@ test_suite* init_unit_test_suite(int, char* []) {
     test->add(BinaryOptionTest::suite());
     test->add(BlackFormulaTest::suite());
     test->add(BondTest::suite());
+    test->add(BondForwardTest::suite());
     test->add(BrownianBridgeTest::suite());
     test->add(BusinessDayConventionTest::suite());
     test->add(CalendarTest::suite());
@@ -380,6 +384,7 @@ test_suite* init_unit_test_suite(int, char* []) {
     test->add(CashFlowsTest::suite());
     test->add(CliquetOptionTest::suite());
     test->add(CmsTest::suite());
+    test->add(ConvertibleBondTest::suite());
     test->add(CovarianceTest::suite());
     test->add(CPISwapTest::suite());
     test->add(CreditDefaultSwapTest::suite());
@@ -444,6 +449,7 @@ test_suite* init_unit_test_suite(int, char* []) {
     test->add(OperatorTest::suite());
     test->add(OptimizersTest::suite(speed));
     test->add(OptionletStripperTest::suite());
+    test->add(OvernightIndexedCouponTest::suite());
     test->add(OvernightIndexedSwapTest::suite());
     test->add(PathGeneratorTest::suite());
     test->add(PeriodTest::suite());
@@ -483,6 +489,7 @@ test_suite* init_unit_test_suite(int, char* []) {
     test->add(AmortizingBondTest::suite());
     test->add(AsianOptionTest::experimental(speed));
     test->add(BasismodelsTest::suite());
+    test->add(BasisSwapRateHelpersTest::suite());
     test->add(BarrierOptionTest::experimental());
     test->add(DoubleBarrierOptionTest::experimental(speed));
     test->add(BlackDeltaCalculatorTest::suite());
@@ -495,7 +502,6 @@ test_suite* init_unit_test_suite(int, char* []) {
     test->add(CommodityUnitOfMeasureTest::suite());
     test->add(CompiledBoostVersionTest::suite());
     test->add(CompoundOptionTest::suite());
-    test->add(ConvertibleBondTest::suite());
     test->add(CreditRiskPlusTest::suite());
     test->add(DoubleBarrierOptionTest::suite(speed));
     test->add(DoubleBinaryOptionTest::suite());
